package com.androidnative.gcm;

import java.io.IOException;

import android.annotation.SuppressLint;
import android.content.SharedPreferences;
import android.content.SharedPreferences.Editor;
import android.os.AsyncTask;
import android.util.Log;

import com.androidnative.gms.utils.AnUtility;
import com.google.android.gms.gcm.GoogleCloudMessaging;
import com.parse.Parse;
import com.parse.ParseException;
import com.parse.ParseInstallation;
import com.parse.ParsePush;
import com.parse.SaveCallback;
import com.unity3d.player.UnityPlayer;

public class ANCloudMessageService {

	public static final String ICON_NAME 		= "ICON_NAME";
	public static final String SOUND_NAME 		= "SOUND_NAME";
	public static final String VIBRATION 		= "VIBRATION";
	public static final String SHOW_WHEN_APP_FOREGROUND = "SHOW_WHEN_APP_FOREGROUND";
	public static final String REPLACE_OLD_WITH_NEW = "REPLACE_OLD_WITH_NEW";
	public static final String USE_PARSE		= "USE_PARSE";
	public static final String PN_PREFS_KEY 	= "AN_PushNotificationBundle";
	
	public static final String PROPERTY_MESSAGE 			= "property_message";
	public static final String MESSAGE_SERVICE_LISTNER_NAME = "GoogleCloudMessageService";

	private GoogleCloudMessaging gcm;
	private String regid;

	private String SENDER_ID = "Your-Sender-ID";

	private static ANCloudMessageService _inctance = null;

	/**
	 * Tag used on log messages.
	 */
	static final String TAG = "AndroidNative";

	public static ANCloudMessageService GetInstance() {
		if (_inctance == null) {
			_inctance = new ANCloudMessageService();
		}

		return _inctance;
		
		
	}

	public ANCloudMessageService() {
		gcm = GoogleCloudMessaging.getInstance(AnUtility.GetLauncherActivity());
	}
	
	public void InitNotificationParams(String icon, String sound, boolean vibration, boolean showWhenAppForeground, boolean replaceOldWithNew) {
		SharedPreferences prefs = AnUtility.GetLauncherActivity().getSharedPreferences(PN_PREFS_KEY, 0);
		Editor editor = prefs.edit();
		editor.putString(ICON_NAME, icon);
		editor.putString(SOUND_NAME, sound);
		editor.putBoolean(VIBRATION, vibration);
		editor.putBoolean(SHOW_WHEN_APP_FOREGROUND, showWhenAppForeground);
		editor.putBoolean(REPLACE_OLD_WITH_NEW, replaceOldWithNew);
		editor.commit();
	}

	@SuppressLint("NewApi")
	public void LoadLastMessage() {
		SharedPreferences mSharedPreferences = AnUtility.GetLauncherActivity().getApplicationContext().getSharedPreferences("MyPref", 0);
   	 	String MessgaeJSON = mSharedPreferences.getString(PROPERTY_MESSAGE, "");
   	 	UnityPlayer.UnitySendMessage(MESSAGE_SERVICE_LISTNER_NAME, "OnLastMessageLoaded", MessgaeJSON);	
	}
	
	public void RemoveLastMessageInfo() {
		SharedPreferences mSharedPreferences = AnUtility.GetLauncherActivity().getApplicationContext().getSharedPreferences("MyPref", 0);
		Editor editor = mSharedPreferences.edit();
		editor.remove(PROPERTY_MESSAGE);
		editor.commit();
	}
	
	

	public void registerDevice(String senderId) {
		if (AnUtility.checkPlayServices()) {
			SENDER_ID = senderId;
			registerInBackground();
		} else {
			Log.i(TAG, "No valid Google Play Services APK found.");
			UnityPlayer.UnitySendMessage(MESSAGE_SERVICE_LISTNER_NAME,
					"OnRegistrationFailed", "");
		}
		
	}
	
	public void initParsePushNotifications(String appId, String dotNetKey) {
		//Initialize Parse PushNotifications support		
		try {
			try {
				Parse.initialize(AnUtility.GetApplicationContex(), appId, dotNetKey);
				ParseInstallation.getCurrentInstallation().saveInBackground();
				ParsePush.subscribeInBackground("", new SaveCallback() {
					  @Override
					  public void done(ParseException e) {
						SharedPreferences prefs = AnUtility.GetLauncherActivity().getSharedPreferences(PN_PREFS_KEY, 0);
						Editor editor = prefs.edit();
					    if (e == null) {					    	
							editor.putBoolean(USE_PARSE, true);
					    	Log.d(TAG, "Successfully subscribed to Parse broadcast Push Notifications");
					    } else {
					    	editor.putBoolean(USE_PARSE, false);
					    	Log.e(TAG, "Parse subscription error: " + e.getMessage());
					    }
					    editor.commit();
					}
				});
				
				Log.i(TAG, "Parse PushNotifications initialized successfully");
			} catch (Exception e) {
				SharedPreferences prefs = AnUtility.GetLauncherActivity().getSharedPreferences(PN_PREFS_KEY, 0);
				Editor editor = prefs.edit();
				editor.putBoolean(USE_PARSE, false);
				editor.commit();
				
				Log.e(TAG, "Failed to initialize Parse Push Notifications " + e.getMessage());
			}
		} catch (NoClassDefFoundError ex) {
			SharedPreferences prefs = AnUtility.GetLauncherActivity().getSharedPreferences(PN_PREFS_KEY, 0);
			Editor editor = prefs.edit();
			editor.putBoolean(USE_PARSE, false);
			editor.commit();
			
			Log.d(TAG, "AndroidNative Parse NoClassDefFoundError " + ex.getMessage());
		}
	}

	/**
	 * Registers the application with GCM servers asynchronously.
	 * <p>
	 * Stores the registration ID and the app versionCode in the application's
	 * shared preferences.
	 */
	private void registerInBackground() {
		new AsyncTask<Void, Void, String>() {
			@Override
			protected String doInBackground(Void... params) {
				String msg = "";
				try {
					regid = gcm.register(SENDER_ID);
					msg = "Registration ID = " + regid;

					// You should send the registration ID to your server over
					// HTTP, so it
					// can use GCM/HTTP or CCS to send messages to your app.
					sendRegistrationIdToBackend();

				} catch (IOException ex) {
					msg = "Error :" + ex.getMessage();
					Log.i(TAG, "registerInBackground error.");
					// If there is an error, don't just keep trying to register.
					// Require the user to click a button again, or perform
					// exponential back-off.

					UnityPlayer.UnitySendMessage(MESSAGE_SERVICE_LISTNER_NAME,
							"OnRegistrationFailed", "");
				}
				return msg;
			}

			@Override
			protected void onPostExecute(String msg) {
				Log.i(TAG, "GCM: SENDER ID: " + SENDER_ID);
				Log.i(TAG, "GCM: " + msg);
			}
		}.execute(null, null, null);
	}

	/**
	 * Sends the registration ID to your server over HTTP, so it can use
	 * GCM/HTTP or CCS to send messages to your app. Not needed for this demo
	 * since the device sends upstream messages to a server that echoes back the
	 * message using the 'from' address in the message.
	 */
	private void sendRegistrationIdToBackend() {
		UnityPlayer.UnitySendMessage(MESSAGE_SERVICE_LISTNER_NAME,
				"OnRegistrationReviced", regid);
	}
}
