package com.androidnative.gms.network;

import java.util.ArrayList;
import java.util.Arrays;
import java.util.List;

import android.annotation.SuppressLint;
import android.app.Activity;
import android.content.Intent;
import android.os.Bundle;
import android.util.Log;

import com.androidnative.gms.core.GameClientManager;
import com.androidnative.gms.core.GooglePlaySupportActivity;
import com.androidnative.gms.core.NewGameHelper;
import com.androidnative.gms.listeners.network.AN_RealTimeMessageReceivedListener;
import com.androidnative.gms.listeners.network.AN_ReliableMessageSentCallback;
import com.androidnative.gms.listeners.network.AN_RoomStatusUpdateListener;
import com.androidnative.gms.listeners.network.AN_RoomUpdateListener;
import com.google.android.gms.games.Games;
import com.google.android.gms.games.GamesActivityResultCodes;
import com.google.android.gms.games.GamesStatusCodes;
import com.google.android.gms.games.Player;
import com.google.android.gms.games.multiplayer.Multiplayer;
import com.google.android.gms.games.multiplayer.Participant;
import com.google.android.gms.games.multiplayer.realtime.Room;
import com.google.android.gms.games.multiplayer.realtime.RoomConfig;
import com.google.android.gms.games.multiplayer.turnbased.TurnBasedMatch;
import com.unity3d.player.UnityPlayer;

public class RealTimeMultiplayerController {
	
	
	protected NewGameHelper mHelper;

	private static RealTimeMultiplayerController _instance = null;
	public Room currentRoom;  
	
	
	private static int MatchVeriant = TurnBasedMatch.MATCH_VARIANT_DEFAULT;
	private static int ExclusiveBitMask = 0;
	
	private boolean _isRealTimeActive = false;
	

	public static RealTimeMultiplayerController GetInstance() {
		if (_instance == null) {
			_instance = new RealTimeMultiplayerController();
		}

		return _instance;
	}
	
	
	// --------------------------------------
	// DELAGETD FROM GCM
	// --------------------------------------
	
	public boolean isRealTimeActive() {
		return _isRealTimeActive;
	}
	
	public void SetGameHelper(NewGameHelper helper) {
		mHelper  = helper;
	}
	
	public void SetVariant(int val) {
		MatchVeriant = val;
	}
	
	
	//will work only if startSelectOpponentsView minPlayers > 0
	public void SetExclusiveBitMask(int val) {
		ExclusiveBitMask = val;
	}
	
    
	
	public void OnStop() {
		//leaveRoom("OnGameCanceled");
	}
	
	public void onActivityResult(int request, int response, Intent data) {
		switch (request) {
        case GameClientManager.RC_SELECT_PLAYERS:        	
        	handleSelectPlayersResult(response, data);
        	break;
        case GameClientManager.RC_WAITING_ROOM:
        	
        	switch(response) {
	        	case Activity.RESULT_OK: 
	        	case Activity.RESULT_CANCELED:
	        		
	        		UnityPlayer.UnitySendMessage(GameClientManager.GOOGLE_PLAY_RTM_LISTENER, "OnWatingRoomIntentClosed", String.valueOf(request) + GameClientManager.UNITY_SPLITTER + String.valueOf(response) );
	        		break;
	        		
	        	case GamesActivityResultCodes.RESULT_LEFT_ROOM:
	        		leaveRoom();
	        		break;
        	}
        
        	break;
		}
	}
	
	
	
	// --------------------------------------
	// PUBLIC METHODS
	// --------------------------------------
	
	@SuppressLint("NewApi")
	public void invitePlayers(int minPlayers, int maxPlayers) {
		Intent intent = Games.RealTimeMultiplayer.getSelectOpponentsIntent(mHelper.getGoogleApiClient(), minPlayers, maxPlayers);
		GooglePlaySupportActivity.startProxyForResult(intent, GameClientManager.RC_SELECT_PLAYERS);
	}
	
	@SuppressLint("NewApi")
	public void showInvitationBox() {
		Intent intent = Games.Invitations.getInvitationInboxIntent(mHelper.getGoogleApiClient());
		GooglePlaySupportActivity.startProxyForResult(intent, GameClientManager.RTM_INBOX_RESULT);
	}
	
	
	public void findMatch(int minPlayers, int maxPlayers, String[] playersToInvite) {
		Log.d(GameClientManager.TAG, "GAME_CLIENT_MANAGER findMatch!!!");
		Bundle autoMatchCriteria = RoomConfig.createAutoMatchCriteria(minPlayers, maxPlayers, ExclusiveBitMask);
		RoomConfig.Builder roomConfigBuilder = makeBasicRoomConfigBuilder();
		roomConfigBuilder.setAutoMatchCriteria(autoMatchCriteria);
		roomConfigBuilder.setVariant(MatchVeriant);
		
		ArrayList<String> ids =  new ArrayList<String>(Arrays.asList(playersToInvite));
		roomConfigBuilder.addPlayersToInvite(ids);		
		
		Games.RealTimeMultiplayer.create(mHelper.getGoogleApiClient(), roomConfigBuilder.build());
	}
	
	public void findMatch(String[] playersToInvite) {
		Log.d(GameClientManager.TAG, "GAME_CLIENT_MANAGER findMatch!!!");
		RoomConfig.Builder roomConfigBuilder = makeBasicRoomConfigBuilder();
		roomConfigBuilder.setAutoMatchCriteria(null);
		roomConfigBuilder.setVariant(MatchVeriant);
		
		ArrayList<String> ids =  new ArrayList<String>(Arrays.asList(playersToInvite));
		roomConfigBuilder.addPlayersToInvite(ids);		
		
		Games.RealTimeMultiplayer.create(mHelper.getGoogleApiClient(), roomConfigBuilder.build());
	}
	
	private RoomConfig.Builder makeBasicRoomConfigBuilder() {
		return RoomConfig.builder(new AN_RoomUpdateListener()).setMessageReceivedListener(new AN_RealTimeMessageReceivedListener()).setRoomStatusUpdateListener(new AN_RoomStatusUpdateListener());
	}
	
	@SuppressLint("NewApi")
	public void showWaitingRoomIntent() {
		// minimum number of players required for our game
		// For simplicity, we require everyone to join the game before we start
		// it (this is signaled by Integer.MAX_VALUE).
		final int MIN_PLAYERS = Integer.MAX_VALUE;
		Intent data = Games.RealTimeMultiplayer.getWaitingRoomIntent(mHelper.getGoogleApiClient(), currentRoom, MIN_PLAYERS);

		// show waiting room UI
		GooglePlaySupportActivity.startProxyForResult(data, GameClientManager.RC_WAITING_ROOM);
	}
	
	
	// --------------------------------------
	// PRIVATE METHODS
	// --------------------------------------
	
	
	public void sendDataToAll(String data, int sendType, int dataId) {
		byte[] stateData = ConvertStringToMultiplayerData(data);
				
		sendMessage(stateData, currentRoom.getParticipants(), sendType, dataId);
	}

	public void sendDataToPlayers(String data, String players, int sendType, int dataId) {
		byte[] stateData = ConvertStringToMultiplayerData(data);
		
		
		
		ArrayList<Participant> participants =  new ArrayList<Participant>();
		
		if(players != null && players.length() > 0) {
			for(String id : players.split(GameClientManager.UNITY_SPLITTER)) {
				
				for (Participant p : currentRoom.getParticipants()) {
					if(p.getPlayer().getPlayerId().equals(id)) {
						participants.add(p);
					}
				}
			}
		}
			
		
		sendMessage(stateData, participants, sendType, dataId);
		
	}
	
	
	private void sendMessage(byte[] data, ArrayList<Participant> participants, int sendType, int dataId) {
		
		if(currentRoom == null) {
			return;
		}

		Player player = Games.Players.getCurrentPlayer(mHelper.getGoogleApiClient());
		
		for (Participant p : currentRoom.getParticipants()) {
			if (p.getParticipantId().equals(player.getPlayerId()))
				continue;
			if (p.getStatus() != Participant.STATUS_JOINED)
				continue;
			
			if(sendType == 0) {
				int messageTokenId = Games.RealTimeMultiplayer.sendReliableMessage(mHelper.getGoogleApiClient(), new AN_ReliableMessageSentCallback(currentRoom.getRoomId(), dataId), data, currentRoom.getRoomId(), p.getParticipantId());
				
				StringBuilder builder = new StringBuilder();
				builder.append(Integer.toString(GamesStatusCodes.STATUS_OK));
				builder.append(GameClientManager.UNITY_SPLITTER);
				builder.append(currentRoom.getRoomId());
				builder.append(GameClientManager.UNITY_SPLITTER);
				builder.append(Integer.toString(messageTokenId));
				builder.append(GameClientManager.UNITY_SPLITTER);
				builder.append(Integer.toString(dataId));
				
				UnityPlayer.UnitySendMessage(GameClientManager.GOOGLE_PLAY_RTM_LISTENER, "OnReliableMessageSent", builder.toString());
			} else {
				Games.RealTimeMultiplayer.sendUnreliableMessage(mHelper.getGoogleApiClient(), data, currentRoom.getRoomId(), p.getParticipantId());
			}			
		}
	}
	
	
	
	// --------------------------------------
	// EVENTS
	// --------------------------------------
	
	public void OnRoomUpdated(Room room) {
		Log.d(GameClientManager.TAG, "OnRoomUpdated total players:" + room.getParticipantIds().size());
		currentRoom = room;
		_isRealTimeActive = true;
		
		StringBuilder info = new StringBuilder();
		
		info.append(room.getRoomId() );
		info.append(GameClientManager.UNITY_SPLITTER);
		
		info.append(room.getCreatorId() );
		info.append(GameClientManager.UNITY_SPLITTER);
		
		String ParticipantsInfo = "";
		
		
		for(Participant participant : room.getParticipants()) {

		
			ParticipantsInfo += participant.getParticipantId() +  ",";

			if(participant.getPlayer() != null) {
				ParticipantsInfo +=  participant.getPlayer().getPlayerId() + ",";
				GameClientManager.GetInstance().loadPlayerInfo(participant.getPlayer().getPlayerId());
			} else {
				ParticipantsInfo += "-1" + ",";
			}
			ParticipantsInfo += String.valueOf(participant.getStatus()) + ",";
			ParticipantsInfo += participant.getHiResImageUrl() + ",";
			ParticipantsInfo += participant.getIconImageUrl() + ",";
			ParticipantsInfo += participant.getDisplayName() + ",";
			
		}
		
		
		ParticipantsInfo += GameClientManager.UNITY_EOF;
		
		info.append(ParticipantsInfo);
		info.append(GameClientManager.UNITY_SPLITTER);

		info.append(room.getStatus());
		info.append(GameClientManager.UNITY_SPLITTER);
		info.append(room.getCreationTimestamp());
		info.append(GameClientManager.UNITY_SPLITTER);
		
				
		
		UnityPlayer.UnitySendMessage(GameClientManager.GOOGLE_PLAY_RTM_LISTENER, "OnRoomUpdate", info.toString());
		
		
	}
	
	public void leaveRoom() {
		if (currentRoom != null) {
			Games.RealTimeMultiplayer.leave(mHelper.getGoogleApiClient(), new AN_RoomUpdateListener(), currentRoom.getRoomId());
			currentRoom = null;
			_isRealTimeActive = false;
		}	
		
	}
	

	public void DismissInvitation(String invitationId) {
		Games.TurnBasedMultiplayer.dismissInvitation(GameClientManager.API(), invitationId);
	}

	public void DeclineInvitation(String invitationId) {
		Games.RealTimeMultiplayer.declineInvitation(GameClientManager.API(), invitationId);
	}

	
	// / Accept the given invitation.
	public void acceptInviteToRoom(String invId) {
		
		Log.d(GameClientManager.TAG, "acceptInviteToRoom+");
		
		RoomConfig.Builder roomConfigBuilder = makeBasicRoomConfigBuilder();
		roomConfigBuilder.setInvitationIdToAccept(invId);
		Games.RealTimeMultiplayer.join(mHelper.getGoogleApiClient(), roomConfigBuilder.build());
	}
		
	
	private void handleSelectPlayersResult(int response, Intent data) {
		if (response == Activity.RESULT_OK) {
			
			Log.d(GameClientManager.TAG, "handleSelectPlayersResult+");

			// get the invite list 
			final ArrayList<String> invitees = data.getStringArrayListExtra(Games.EXTRA_PLAYER_IDS);
		
			Log.d(GameClientManager.TAG, "invitees: " + invitees.size());
			// get the automatch criteria
			Bundle autoMatchCriteria = null;
			int minAutoMatchPlayers = data.getIntExtra(Multiplayer.EXTRA_MIN_AUTOMATCH_PLAYERS, 0);
			int maxAutoMatchPlayers = data.getIntExtra(Multiplayer.EXTRA_MAX_AUTOMATCH_PLAYERS, 0);
			
			if (minAutoMatchPlayers > 0 || maxAutoMatchPlayers > 0) {
				autoMatchCriteria = RoomConfig.createAutoMatchCriteria(minAutoMatchPlayers, maxAutoMatchPlayers, ExclusiveBitMask);
			}
	
			
			RoomConfig.Builder rtmConfigBuilder = makeBasicRoomConfigBuilder();
			rtmConfigBuilder.setVariant(MatchVeriant);
			rtmConfigBuilder.addPlayersToInvite(invitees);
			if (autoMatchCriteria != null) {
				rtmConfigBuilder.setAutoMatchCriteria(autoMatchCriteria);
			}
			
			
			Log.d(GameClientManager.TAG, "create room");
			Games.RealTimeMultiplayer.create(mHelper.getGoogleApiClient(), rtmConfigBuilder.build());

		}
		
		UnityPlayer.UnitySendMessage(GameClientManager.GOOGLE_PLAY_RTM_LISTENER, "OnInvitationBoxUiClosed",  "10001" + GameClientManager.UNITY_SPLITTER  +String.valueOf(response));
	}

	
	private static byte[] ConvertStringToMultiplayerData(String data) {
		String[] array = data.split(",");
		List<Byte> l = new ArrayList<Byte>();
		for (String b : array) {
			int temp_param = Integer.valueOf(b);
			byte byte_value = (byte)temp_param;
			l.add(Byte.valueOf(String.valueOf(byte_value)));
		}

		Byte[] B = l.toArray(new Byte[l.size()]);

		byte[] stateData = new byte[B.length];
		for (int i = 0; i < B.length; i++) {
			stateData[i] = B[i];
		}

		return stateData;
	}

	
	

	
	
}
